@@ -140,6 +140,15 @@ def sort_by_RGB(a, b):
 		return 0
 
 
+def sort_by_BGR(a, b):
+	if a[:3][::-1] > b[:3][::-1]:
+		return 1
+	elif a[:3] == b[:3]:
+		return 0
+	else:
+		return -1
+
+
 def sort_by_RGB_sum(a, b):
 	sum1, sum2 = sum(a[:3]), sum(b[:3])
 	if sum1 > sum2:
@@ -280,6 +289,8 @@ class CGATS(dict):
 			context = self
 
 			for raw_line in raw_lines:
+				# Replace 1.#IND00 with NaN
+				raw_line = raw_line.replace("1.#IND00", "NaN")
 				# strip control chars and leading/trailing whitespace
 				line = re.sub('[^\x09\x20-\x7E\x80-\xFF]', '', 
 								raw_line.strip())
@@ -797,6 +808,9 @@ class CGATS(dict):
 	def sort_by_RGB(self):
 		return self.sort_data_RGB_XYZ(sort_by_RGB)
 	
+	def sort_by_BGR(self):
+		return self.sort_data_RGB_XYZ(sort_by_BGR)
+	
 	def sort_by_RGB_pow_sum(self):
 		return self.sort_data_RGB_XYZ(sort_by_RGB_pow_sum)
 	
@@ -887,7 +901,18 @@ class CGATS(dict):
 													  'got %s)' % 
 													  (item, type(value)))
 							else:
-								lencheck = len(str(abs(value)).split("e")[0])
+								strval = str(abs(value))
+								if (self.parent.type != "CAL" and
+									item.startswith("RGB_") or
+									item.startswith("CMYK_")):
+									# Assuming 0..100, 4 decimal digits is
+									# enough for roughly 19 bits integer
+									# device values
+									parts = strval.split(".")
+									if len(parts) == 2 and len(parts[-1]) > 4:
+										value = round(value, 4)
+										strval = str(abs(value))
+								lencheck = len(strval.split("e")[0])
 								if lencheck > self.vmaxlen:
 									self.vmaxlen = lencheck
 						elif self.root.normalize_fields and \
@@ -1016,7 +1041,7 @@ class CGATS(dict):
 				  normalize_RGB_white=False, compress=True, format="VRML"):
 		if colorspace not in ("DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
 							  "LCH(uv)", "Lab", "Luv", "Lu'v'", "RGB", "xyY",
-							  "HSI", "HSL", "HSV", "ICtCp"):
+							  "HSI", "HSL", "HSV", "ICtCp", "IPT", "Lpt"):
 			raise ValueError("export_3d: Unknown colorspace %r" % colorspace)
 		import x3dom
 		data = self.queryv1("DATA")
@@ -1079,7 +1104,7 @@ Transform {
 		}
 """
 		axes = ""
-		if (colorspace not in ("Lab", "Luv", "ICtCp") and
+		if (colorspace not in ("Lab", "Luv", "ICtCp", "IPT", "Lpt") and
 			not colorspace.startswith("DIN99")):
 			if colorspace in ("Lu'v'", "xyY"):
 				maxz = scale = 100
@@ -1149,10 +1174,23 @@ Transform {
 				nxcolor = "0.8 1.0 0.0"
 				pycolor = "1.0 0.0 0.25"
 				nycolor = "0.0 1.0 1.0"
+			elif colorspace == "IPT":
+				(pxlabel,
+				 nxlabel,
+				 pylabel,
+				 nylabel,
+				 pllabel) = ('"P", "+%.1f"' % 1,
+							 '"P", "-%.1f"' % 1,
+							 '"T +%.1f"' % 1,
+							 '"T -%.1f"' % 1,
+							 '"I"')
 			else:
 				if colorspace == "Luv":
 					x = "u"
 					y = "v"
+				elif colorspace == "Lpt":
+					x = "p"
+					y = "t"
 				else:
 					x = "a"
 					y = "b"
@@ -1344,6 +1382,8 @@ Transform {
 									  entry["XYZ_Y"],
 									  entry["XYZ_Z"],
 									  white,
+									  "D65" if colorspace in ("ICtCp", "IPT")
+									  else "D50",
 									  cat=cat)
 			L, a, b = colormath.XYZ2Lab(X, Y, Z)
 			if colorspace == "RGB":
@@ -1416,6 +1456,12 @@ Transform {
 				I, Ct, Cp = colormath.XYZ2ICtCp(X / 100.0, Y / 100.0, Z / 100.0,
 												clamp=False)
 				x, y, z = Ct * 100, Cp * 100, I * 100 - 50
+			elif colorspace == "IPT":
+				I, P, T = colormath.XYZ2IPT(X / 100.0, Y / 100.0, Z / 100.0)
+				x, y, z = P * 100, T * 100, I * 100 - 50
+			elif colorspace == "Lpt":
+				L, p, t = colormath.XYZ2Lpt(X, Y, Z)
+				x, y, z = p, t, L - 50
 			if RGB_black_offset != 40:
 				# Keep reference hue and saturation
 				# Lab to sRGB using reference black offset of 40 like Argyll CMS
@@ -1601,6 +1647,56 @@ Transform {
 		self.setmodified()
 		return result
 
+	def convert_XYZ_to_Lab(self):
+		"""
+		Convert XYZ to D50 L*a*b* and add it as additional fields
+		
+		"""
+		color_rep = (self.queryv1("COLOR_REP") or "").split("_")
+
+		if color_rep[1] == "LAB":
+			# Nothing to do
+			return
+
+		if (len(color_rep) != 2 or color_rep[0] not in ("RGB", "CMYK") or
+			color_rep[1] != "XYZ"):
+			raise NotImplementedError("Got unsupported color representation %s"
+									  % "_".join(color_rep))
+
+		data = self.queryv1("DATA")
+		if not data:
+			raise CGATSError("No data")
+
+		if color_rep[0] == "RGB":
+			white = data.queryv1({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
+		elif color_rep[0] == "CMYK":
+			white = data.queryv1({"CMYK_C": 0, "CMYK_M": 0, "CMYK_Y": 0,
+								  "CMYK_K": 0})
+		if not white:
+			raise CGATSError("Missing white patch")
+
+		device_labels = []
+		for channel in color_rep[0]:
+			device_labels.append(color_rep[0] + "_" + channel)
+
+		# Always XYZ
+		cie_labels = []
+		for channel in color_rep[1]:
+			cie_labels.append(color_rep[1] + "_" + channel)
+
+		# Add entries to DATA_FORMAT
+		Lab_data_format = ("LAB_L", "LAB_A", "LAB_B")
+		for label in Lab_data_format:
+			if not label in data.parent.DATA_FORMAT.values():
+				data.parent.DATA_FORMAT.add_data((label, ))
+
+		# Add L*a*b* to each sample
+		for key, sample in data.iteritems():
+			cie_values = [sample[label] for label in cie_labels]
+			Lab = colormath.XYZ2Lab(*cie_values)
+			for i, label in enumerate(Lab_data_format):
+				sample[label] = Lab[i]
+
 	def fix_zero_measurements(self, warn_only=False, logfile=safe_print):
 		"""
 		Fix (or warn about) zero measurements
@@ -1693,6 +1789,20 @@ Transform {
 							break
 		return fixed
 	
+	def quantize_device_values(self, bits=8):
+		""" Quantize device values to n bits """
+		q = 2 ** bits - 1.0
+		for data in self.queryv("DATA").itervalues():
+			if data.parent.type == "CAL":
+				maxv = 1.0
+			else:
+				maxv = 100.0
+			color_rep = (data.parent.queryv1("COLOR_REP") or "").split("_")[0]
+			for labels in get_device_value_labels(color_rep):
+				for item in data.queryi(labels).itervalues():
+					for label in labels:
+						item[label] = round(item[label] / maxv * q) / q * maxv
+	
 	def scale_device_values(self, factor=100.0 / 255, color_rep=None):
 		""" Scales device values by multiplying with factor. """
 		for labels in get_device_value_labels(color_rep):
