@@ -391,6 +391,7 @@ class ReportFrame(BaseFrame):
 			else:
 				path = None
 			setcfg("measurement_report.output_profile", path)
+			XYZbpout = self.XYZbpout
 			# XYZbpout will be set to the blackpoint of the selected profile.
 			# This is used to determine if black output offset controls should
 			# be shown. Set a initial value slightly above zero so output
@@ -399,6 +400,7 @@ class ReportFrame(BaseFrame):
 		else:
 			profile = None
 			if which == "input":
+				XYZbpin = self.XYZbpin
 				self.XYZbpin = [0, 0, 0]
 		if path or profile:
 			if path and not os.path.isfile(path):
@@ -432,6 +434,7 @@ class ReportFrame(BaseFrame):
 					if (not getattr(self, which + "_profile", None) or
 						getattr(self, which + "_profile").fileName !=
 						profile.fileName):
+						# Profile selection has changed
 						if which == "simulation":
 							# Get profile blackpoint so we can check if it makes
 							# sense to show TRC type and output offset controls
@@ -466,6 +469,13 @@ class ReportFrame(BaseFrame):
 									self.set_profile_ctrl_path(which)
 									return
 								self.XYZbpout = odata[0]
+					else:
+						# Profile selection has not changed
+						# Restore cached XYZbp values
+						if which == "output":
+							self.XYZbpout = XYZbpout
+						elif which == "input":
+							self.XYZbpin = XYZbpin
 					setattr(self, "%s_profile" % which, profile)
 					if not silent:
 						setcfg("measurement_report.%s_profile" % which,
@@ -521,7 +531,8 @@ class ReportFrame(BaseFrame):
 		shown = self.apply_trc_ctrl.IsShown()
 		enable6 = (shown and
 				   bool(getcfg("measurement_report.apply_trc")))
-		show = shown and self.mr_trc_ctrl.GetSelection() == 2
+		show = shown and (self.mr_trc_ctrl.GetSelection() == 2 or
+						  getcfg("show_advanced_options"))
 		self.panel.Freeze()
 		self.mr_trc_ctrl.Enable(enable6)
 		self.mr_trc_ctrl.Show(shown)
@@ -580,7 +591,7 @@ class ReportFrame(BaseFrame):
 		self.output_profile_ctrl.SetPath(getcfg("measurement_report.output_profile"))
 		self.set_profile("output", silent=True)
 		self.mr_set_testchart(getcfg("measurement_report.chart"))
-		self.use_simulation_profile_ctrl_handler(None)
+		self.use_simulation_profile_ctrl_handler(None, update_trc=False)
 		self.panel.Thaw()
 
 	def mr_update_trc_control(self):
@@ -812,7 +823,7 @@ class ReportFrame(BaseFrame):
 			   int(self.devlink_profile_cb.GetValue()))
 		self.mr_update_main_controls()
 	
-	def use_simulation_profile_ctrl_handler(self, event):
+	def use_simulation_profile_ctrl_handler(self, event, update_trc=True):
 		if event:
 			use_sim_profile = self.simulation_profile_cb.GetValue()
 			setcfg("measurement_report.use_simulation_profile",
@@ -829,28 +840,30 @@ class ReportFrame(BaseFrame):
 				"bTRC" in sim_profile.tags and sim_profile.tags.rTRC ==
 				sim_profile.tags.gTRC == sim_profile.tags.bTRC and
 				isinstance(sim_profile.tags.rTRC, ICCP.CurveType)):
-				tf = sim_profile.tags.rTRC.get_transfer_function()
-				# Use BT.1886 gamma mapping for SMPTE 240M / Rec. 709 TRC
-				setcfg("measurement_report.apply_trc",
-					   int(tf[0][1] in (-240, -709) or
-						   (tf[0][0].startswith("Gamma") and tf[1] >= .95)))
-				# Use only BT.1886 black output offset
-				setcfg("measurement_report.apply_black_offset",
-					   int(tf[0][1] not in (-240, -709) and
-						   (not tf[0][0].startswith("Gamma") or tf[1] < .95) and
-						   self.XYZbpin != self.XYZbpout))
-				# Set gamma to profile gamma if single gamma profile
-				if tf[0][0].startswith("Gamma") and tf[1] >= .95:
-					if not getcfg("measurement_report.trc_gamma.backup", False):
-						# Backup current gamma
-						setcfg("measurement_report.trc_gamma.backup",
-							   getcfg("measurement_report.trc_gamma"))
-					setcfg("measurement_report.trc_gamma", round(tf[0][1], 2))
-				# Restore previous gamma if not single gamma profile
-				elif getcfg("measurement_report.trc_gamma.backup", False):
-					setcfg("measurement_report.trc_gamma",
-						   getcfg("measurement_report.trc_gamma.backup"))
-					setcfg("measurement_report.trc_gamma.backup", None)
+				tf = sim_profile.tags.rTRC.get_transfer_function(outoffset=1.0)
+				if update_trc or self.XYZbpin == self.XYZbpout:
+					# Use only BT.1886 black output offset
+					setcfg("measurement_report.apply_black_offset",
+						   int(tf[0][1] not in (-240, -709) and
+							   (not tf[0][0].startswith("Gamma") or tf[1] < .95) and
+							   self.XYZbpin != self.XYZbpout))
+				if update_trc:
+					# Use BT.1886 gamma mapping for SMPTE 240M / Rec. 709 TRC
+					setcfg("measurement_report.apply_trc",
+						   int(tf[0][1] in (-240, -709) or
+							   (tf[0][0].startswith("Gamma") and tf[1] >= .95)))
+					# Set gamma to profile gamma if single gamma profile
+					if tf[0][0].startswith("Gamma") and tf[1] >= .95:
+						if not getcfg("measurement_report.trc_gamma.backup", False):
+							# Backup current gamma
+							setcfg("measurement_report.trc_gamma.backup",
+								   getcfg("measurement_report.trc_gamma"))
+						setcfg("measurement_report.trc_gamma", round(tf[0][1], 2))
+					# Restore previous gamma if not single gamma profile
+					elif getcfg("measurement_report.trc_gamma.backup", False):
+						setcfg("measurement_report.trc_gamma",
+							   getcfg("measurement_report.trc_gamma.backup"))
+						setcfg("measurement_report.trc_gamma.backup", None)
 				self.mr_update_trc_controls()
 				enable = (tf[0][1] not in (-240, -709) and
 						  self.XYZbpin != self.XYZbpout)
